@@ -141,6 +141,12 @@ ops = {
 }
 
 
+def chunks(lst, n):
+    """Yield successive n-sized chunks from lst."""
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
+
+
 def buildquery(json_object):
     if 'field' in json_object.keys():
         # In this case the json_object is an object which describes a single query
@@ -393,7 +399,10 @@ def trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_ac
     print("Inside trigger_set_value_task")
     id = conf.get('data').get('id', '')
     lead_id = conf.get('data').get('leadid', '')
-    record_module = 'Leads'
+    record_module = conf.get('source_module', '')
+    if not record_module:
+        print("unable to get source module from conf  object")
+        return None
     module = module_id_dict.get(record_module, '')
     print("Id: {}".format(id))
     print("lead_id: {}".format(lead_id))
@@ -788,3 +797,76 @@ def get_crm_records(vtiger_access, conf):
 
     else:
         print("unable to get id & data from conf object")
+
+
+def get_record_details_from_crm(vtiger_access, data):
+    """ This function will prepare list of record ids and will fetch record data for those ids from Vtiger.
+       NOTE: set return value in context["dag_run"].conf['data'] while calling this function.
+       params:
+       vtiger_access: vtiger access dictionary
+       data: data of conf object"""
+
+    if data:
+
+        # get source module
+        source_module = data[0].get('module')
+        if not source_module:
+            print("unable to get source module from data")
+            return []
+
+        module_id = module_id_dict[source_module]
+
+        # prepare list of ids
+        user_id_list = []
+
+        for element in data:
+            id = element.get('id', '')
+            if id:
+                # prepare query id using source module
+                record_id = "{module}x{record_id}".format(module=module_id, record_id=id)
+
+                user_id_list.append(record_id)
+        print("user id list received: ", user_id_list)
+
+        # call vtiger api to get data
+
+        session_name = vtiger_access.get('vtiger_session', '')
+        print("Session name: {}".format(session_name))
+        if not session_name:
+            print("unable to get session id from vtiger access. Please try again")
+            return None
+
+        url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query=SELECT * FROM {source_module} WHERE id in ('.format(
+            vtiger_url=vtiger_access.get('vtiger_url'), sessionName=session_name, source_module=source_module) + ','.join((str(n) for n in user_id_list)) + ');'
+
+        response, status = invoke_http_request(url, 'GET', {'content_type': 'application/json'})
+
+        # request session if existing one is expired
+
+        if response:
+            if not response.get('success', ''):
+                if response.get('error', ''):
+                    if response.get('error').get('code') in ["INVALID_USER_CREDENTIALS", "INVALID_SESSIONID"]:
+                        session_name = get_session_name(vtiger_access)
+
+                        # retry to execute query
+
+                        url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query=SELECT * FROM {source_module} WHERE id in ('.format(
+                            vtiger_url=vtiger_access.get('vtiger_url'), sessionName=session_name,
+                            source_module=source_module) + ','.join((str(n) for n in user_id_list)) + ');'
+
+                        print("url", url)
+                        response, status = invoke_http_request(url, 'GET', {'content_type': 'application/json'})
+
+                        print("response, status", response, status)
+                        vtiger_access['vtiger_session'] = session_name
+                        Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+
+        if 'success' in response and response['success'] and 'result' in response:
+            # return list of user records
+            return response['result']
+        else:
+            print("unable to get record details from Vtiger. Response: ", response.text)
+
+    else:
+        print("unable to get id & data from conf object")
